@@ -1,6 +1,7 @@
 import lxml.etree as ET
 
-from amilib.util import AmiLogger, Util
+from amilib.file_lib import FileLib
+from amilib.util import Util
 from amilib.xml_lib import NS_MAP, XML_NS, SVG_NS
 
 SVG_SVG = "svg"
@@ -37,7 +38,7 @@ FILL = "fill"
 STROKE = "stroke"
 STROKE_WIDTH = "stroke-width"
 
-logger = AmiLogger.create_named_logger(__file__)
+logger = Util.get_logger(__name__)
 
 class AmiSVG:
     """for actually rendering SVG?"""
