@@ -12,6 +12,9 @@ from pathlib import Path
 # i have no idea why html_args and pdf_args behave differently
 from amilib.file_lib import FileLib
 
+from amilib.dict_args import AmiDictArgs
+from amilib.util import Util
+
 try:
     from html_args import HTMLArgs
 except ModuleNotFoundError as e:
@@ -27,7 +30,8 @@ from amilib.wikimedia import WikidataLookup
 AMIX_DIR = Path(__file__).parent
 REPO_DIR = AMIX_DIR.parent
 
-logger = FileLib.get_logger(__file__)
+logger = Util.get_logger(__name__)
+logger.setLevel(logging.INFO)
 
 class AmiLib:
     COPY = "copy"
@@ -54,11 +58,13 @@ class AmiLib:
         """creates adds the arguments for pyami commandline
         """
 
-        def run_dict(self):
-            logger.debug(f"run dict pyamix")
+        def run_dict(selfx):
+            logger.debug(f"run dict amilib")
+            pass
 
         def run_pdf(args):
             logger.debug(f"run pdf")
+            pass
 
 
         version = self.version()
@@ -66,7 +72,7 @@ class AmiLib:
             # what does this do?
             sys.argv = [AmiLib.AmiLib]
         parser = argparse.ArgumentParser(
-            description=f'pyamihtmlx: V{version} call with ONE of subcommands (HTML,PDF, IPCC, UNFCCC), e.g. pyamihtmlx IPCC --help'
+            description=f'amilib: V{version} call with ONE of subcommands (DICT, HTML,PDF), e.g. amilib DICT --help'
         )
 
         parser.add_argument('-v', '--version', action="store_true",
@@ -86,9 +92,9 @@ class AmiLib:
             '\n'
             '\nExamples (# foo is a comment):\n'
             '  amilib        # runs help\n'
-            '  pyamihtmlx -h     # runs help\n'
-            '  pyamihtmlx PDF -h # runs PDF help\n'
-            '  pyamihtmlx PDF --infile foo.pdf --outdir bar/ # converts PDF to HTML\n'
+            '  amilib -h     # runs help\n'
+            '  amilib PDF -h # runs PDF help\n'
+            '  amilib PDF --infile foo.pdf --outdir bar/ # converts PDF to HTML\n'
             '\n'
             '----------------------------------------\n\n'
         )
@@ -97,17 +103,20 @@ class AmiLib:
 
         subparsers = parser.add_subparsers(help='subcommands', dest="command")
         for choice in subparsers.choices:
-            print(f">>> {choice}")
+            logger.debug(f">>> {choice}")
+            pass
 
         pdf_parser = AmiLibArgs.make_sub_parser(PDFArgs(), subparsers)
         logger.debug(f"pdf_parser {pdf_parser}")
         html_parser = AmiLibArgs.make_sub_parser(HTMLArgs(), subparsers)
         logger.debug(f"html_parser {html_parser}")
+        dict_parser = AmiLibArgs.make_sub_parser(AmiDictArgs(), subparsers)
+        logger.debug(f"dict_parser {dict_parser}")
 
         parser.epilog = "other entry points run as 'python -m amilib.amix <args>'"
         parser.epilog = """run:
         pyamihtmlx <subcommand> <args>
-          where subcommand is in   {HTML,PDF} and args depend on subcommand
+          where subcommand is in   {DICT, HTML,PDF} and args depend on subcommand
         """
 
         return parser
@@ -140,16 +149,20 @@ class AmiLib:
         """parses cmdline, runs command and outputs symbols
 
         :param args: either a string or a list of strings
+        NOTE: if any arg in noty a string it is transformed into one
 
         if args is a string we split it at spaces into a list of strings
 
         """
         logger.debug(f"********** raw arglist {args}")
+        # split space_separated string into strings
         if isinstance(args, str):
             args = args.strip()
             args = args.split(" ")
 
-        logger.warning(f"command: {args}")
+        # convert all args to str
+        args = [str(s) for s in args]
+        logger.info(f"command: {args}")
         test_catch = False
         if test_catch:  # try to trap exception
             try:
@@ -189,7 +202,6 @@ class AmiLib:
         May duplicates
         """
         new_items = {}
-        # self.logger.debug(f"SYMBOLS1 {self.symbol_ini.symbols}")
         for item in self.args.items():
             new_item = self.make_substitutions(item)
             logger.debug(f"++++++++{item} ==> {new_item}")
@@ -227,14 +239,14 @@ class AmiLib:
         logging.debug(f" COMMAND: {subparser_type} {self.args}")
 
         subparser_dict = {
-            # "DICT": AmiDictArgs(),
+            "DICT": AmiDictArgs(),
             # "GUI":  GUIArgs(),
             "HTML": HTMLArgs(),
             "PDF": PDFArgs(),
         }
         abstract_args = subparser_dict.get(subparser_type)
 
-        logger.warning(f"abstract_args {abstract_args}")
+        logger.debug(f"abstract_args {abstract_args}")
         if abstract_args:
             abstract_args.parse_and_process1(self.args)
         else:
@@ -242,7 +254,7 @@ class AmiLib:
 
     #
     def run_core_mathods(self):
-        logging.debug(f"run_core")
+        # logging.debug(f"run_core")
         # mainly obsolete
         if self.VERSION in self.args and self.args[self.VERSION] is not None:
             self.print_version()
@@ -250,6 +262,7 @@ class AmiLib:
     def print_version(self):
         print(f"version {self.version()}")
 
+
     #
     def replace_single_values_in_self_args_with_list(self, key):
         """always returns list even for single arg
@@ -374,6 +387,19 @@ class AmiLib:
         version = '0.1.3'  # 2024-05-20 # revert pdfplumber to 0.10.0
         version = '0.1.4'  # 2024-05-22 # revert pdfplumber to 0.11.0 
         version = '0.1.5'  # 2024-05-25 # fixed nlp, pdfplumber
+        version = '0.2.0a1'  # 2024-06-06 # includes amidict 
+        version = '0.2.1a1'  # 2024-06-06 # includes amidict and commandline
+        version = '0.2.1a2'  # 2024-06-06 # includes amidict and commandline
+        version = '0.2.1a3'  # 2024-07-03 # added wordlists
+        version = '0.2.2a1'  # 2024-07-11 # wikipedia lookup and paragrah splitting
+        version = '0.2.3a1'  # 2024-07-16 # markup html with dictionaries
+        version = '0.2.3a2'  # 2024-07-17 # markup html with dictionaries
+        version = '0.2.4a1'  # 2024-07-17 # add Wiktionary
+        version = '0.2.4a2'  # 2024-08-27 # build dictionaries from wordlists preparing for release
+        version = '0.2.5a1'  # 2024-08-30 # fixed Wiktionary bug
+        version = '0.2.5'    # 2024-09-09 # testing as library usable by amiclimate
+        version = '0.2.6'    # 2024-09-12 # added medisawiki parser
+        version = '0.2.7'    # 2024-09-12 # corrected bug
 
         # logging.warn(f"VERSION {version}")
         return version
@@ -402,7 +428,7 @@ class AmiLibArgs(AbstractArgs):
         """adds arguments to a parser or subparser"""
         self.parser.description = 'Abstract editing analysing annotation'
         self.parser.add_argument(f"--foo", action="store_true",
-                                 help="annotate HTML file with dictionary")
+                                 help="annotate HTML file with dictionary (NYI, TEST)")
         # self.parser.add_argument(f"--{COLOR}", type=str, nargs=1,
         #                          help="colour for annotation")
         # self.parser.add_argument(f"--{DICT}", type=str, nargs=1,
@@ -413,7 +439,7 @@ class AmiLibArgs(AbstractArgs):
         #                          help="output html file")
         # self.parser.add_argument(f"--{OUTDIR}", type=str, nargs=1,
         #                          help="output directory")
-        self.parser.epilog = "=======abstract add arguments ========"
+        self.parser.epilog = "======= ========"
 
     """python -m pyamihtmlx.pyamix HTML --annotate 
      --dict /Users/pm286/projects/semanticClimate/ipcc/ar6/wg3/Chapter02/dict/emissions_abbreviations.xml
@@ -468,16 +494,16 @@ class AmiLibArgs(AbstractArgs):
         if not self.inpath:
             logging.error(f"no input file to annotate given")
             return
-        if not self.outpath:
+        if not self.location_xml:
             logging.error(f"no output file given")
             return
         if not self.outdir:
-            self.outdir = Path(self.outpath).parent
+            self.outdir = Path(self.location_xml).parent
         self.outdir = Path(self.outdir)
         self.outdir.mkdir(exist_ok=True)
 
         self.ami_dict = AmiDictionary.create_from_xml_file(self.dictfile)
-        self.ami_dict.markup_html_from_dictionary(self.inpath, self.outpath, self.color)
+        self.ami_dict.markup_html_from_dictionary(self.inpath, self.location_xml, self.color)
 
     def make_foo(self):
         pass
