@@ -1,6 +1,7 @@
 # TODO cyclic imports with html
 import argparse
 import logging
+import os
 import re
 import sys
 import textwrap
@@ -9,9 +10,11 @@ from pathlib import Path
 
 import lxml.etree as ET
 
+from amilib.ami_args import AmiArgParser, AbstractArgs
 from amilib.ami_html import A_HREF, H_A, H_SPAN, HtmlTidy, HtmlStyle, CSSStyle
 from amilib.ami_pdf_libs import AmiPage, PDFParser, DEBUG_OPTIONS
-from amilib.util import AbstractArgs, AmiArgParser, Util, AmiLogger
+from amilib.util import Util
+from amilib.file_lib import FileLib
 
 INDIR = "indir"
 INFILE = "infile"
@@ -29,21 +32,6 @@ MAXPAGE = "maxpage"
 PAGES = "pages"
 PDF2HTML = "pdf2html"
 
-# ANNOTS = "annots"
-# CURVES = "curves"
-# HYPERLINKS = "hyperlinks"
-# IMAGES = "images"
-# LINES = "lines"
-# PTS = 'pts'
-# RECTS = "rects"
-# TABLES = "tables"
-# TEXTS = "texts"
-# WORDS = "words"
-#
-# DEFAULT_CONVERT = "html"
-# DEBUG_OPTIONS = [WORDS, LINES, RECTS, CURVES, IMAGES, TABLES, HYPERLINKS, TEXTS, ANNOTS]
-# DEBUG_ALL = "debug_all"
-
 MAX_MAXPAGE = 9999999
 CONVERT = "convert"
 FLOW = "flow"
@@ -53,12 +41,13 @@ HEADER = "header"
 DEFAULT_CONVERT = "html"
 DEFAULT_MAXPAGES = 100
 
-logger = AmiLogger.create_named_logger(__file__)
+logger = Util.get_logger(__name__)
+logger.setLevel(logging.INFO)
 
 
 class PDFArgs(AbstractArgs):
     """
-    Holds argument values for pyamihtmlx PDF commands and runs conversions
+    Holds argument values for pyami PDF commands and runs conversions
     Also holds much of the document data
 
         self.convert = DEFAULT_CONVERT
@@ -90,6 +79,8 @@ class PDFArgs(AbstractArgs):
 
     def __init__(self):
         """arg_dict is set to default"""
+        logger.debug("creating PDFArgs")
+
         super().__init__()
         self.convert = DEFAULT_CONVERT
         self.html = None
@@ -122,12 +113,10 @@ class PDFArgs(AbstractArgs):
         """creates adds the arguments for pyami commandline
 
         """
+        logger.debug(f"================== add arguments PDF ================")
         if self.parser is None:
-            # self.parser = argparse.ArgumentParser(
-            #     usage="pyamihtmlx always uses subcommands (DICT,GUI,HTML,PDF,PROJECT)\n e.g. pyamihtmlx PDF --help"
-            # )
             self.parser = AmiArgParser(
-                usage="pyamihtmlx always uses subcommands (DICT,GUI,HTML,PDF,IPCC,PROJECT)\n e.g. pyamihtmlx PDF --help"
+                usage="PDF: amilib always uses subcommands (HTML,PDF)\n e.g. amilib PDF --help"
             )
 
         self.parser.description = textwrap.dedent(
@@ -139,7 +128,6 @@ class PDFArgs(AbstractArgs):
             '  * PDF --help\n'
         )
         self.parser.formatter_class = argparse.RawDescriptionHelpFormatter
-        # self.parser.add_argument("--convert", type=str, choices=[], help="conversions (NYI)")
         self.parser.add_argument("--debug", type=str, choices=DEBUG_OPTIONS, help="debug these during parsing (NYI)")
         self.parser.add_argument("--flow", type=bool, nargs=1,
                                  help="create flowing HTML, e.g. join lines, pages (heuristics)", default=True)
@@ -177,6 +165,7 @@ class PDFArgs(AbstractArgs):
         self.parser.add_argument("--resolution", type=int, nargs=1, help="resolution of output images (if imagedir)",
                                  default=400)
         self.parser.add_argument("--template", type=str, nargs=1, help="file to parse specific type of document (NYI)")
+        self.parser.epilog = "============ PDF epilog ==========="
         return self.parser
 
     # class PDFArgs:
@@ -195,13 +184,13 @@ class PDFArgs(AbstractArgs):
                         resolution of output images
   --template TEMPLATE   file to parse specific type of document"""
 
+        logger.debug(f"running PDFArgs.process_args()")
         if self.arg_dict:
             #            logging.warning(f"ARG DICTXX {self.arg_dict}")
             self.read_from_arg_dict()
 
         if not self.check_input():
-            # self.parser.print_help() # self.parser is null
-            print("for help, run 'pyamihtmlx PDF -h'")
+            logger.debug("for help, run 'pyamihtmlx PDF -h'")
             return
         self.create_consistent_output_filenames_and_dirs()
         self.calculate_headers_footers()
@@ -214,7 +203,7 @@ class PDFArgs(AbstractArgs):
             outdir = self.arg_dict.get(OUTDIR)
             outpath = self.arg_dict.get(OUTPATH)
             if outdir is None and outpath is not None:
-                outdir = outpath.parent
+                outdir = Path(outpath).parent
             if outpath is None:
                 if outdir is None:
                     raise FileNotFoundError(f"no outdir or outpath given")
@@ -241,7 +230,7 @@ class PDFArgs(AbstractArgs):
 
     def check_input(self):
         if not self.inpath:
-            print(f"No input file, no action taken")
+            logger.debug(f"No input file, no action taken")
             return False
             # raise FileNotFoundError(f"input file not given")
         if not Path(self.inpath).exists():
@@ -371,9 +360,9 @@ class PDFArgs(AbstractArgs):
         if self.flow runs self.tidy_flow
         :return: outpath
         """
-        print(f"flow {flow} indir {indir} inpath {inpath} maxpage {maxpage} outform {outform} \n"
+        logger.debug(f"flow {flow} indir {indir} inpath {inpath} maxpage {maxpage} outform {outform} \n"
               f"outpath {outpath} outstem {outstem} outdir {outdir} pdf2html {pdf2html} process_args {process_args}")
-        print(f"==============CONVERT================")
+        logger.debug(f"==============CONVERT================")
         # process arguments into a dictionary
         if flow:
             self.arg_dict[FLOW] = flow
@@ -418,12 +407,12 @@ class PDFArgs(AbstractArgs):
         if out_html is None:
             raise ValueError(f" out_html is None")
         if outpath is None:
-            print(f"no outpath given")
+            logger.debug(f"no outpath given")
             return None, None
         outpath1 = str(outpath)
         with Util.open_write_utf8(outpath1) as f:
             f.write(out_html)
-            print(f"wrote partially tidied html {outpath}")
+            logger.debug(f"wrote partially tidied html {outpath}")
         return outpath, out_html
     @classmethod
     def pdf_to_raw_then_raw_to_tidy(
@@ -528,7 +517,7 @@ class PDFArgs(AbstractArgs):
 
         result = re.compile(refregex).search(text)
         if result:
-            # print(f"matched: {result.group(1)} {result.group(2)}, {result.group(3)} {result.groups()}")
+            # logger.debug(f"matched: {result.group(1)} {result.group(2)}, {result.group(3)} {result.groups()}")
             elem0 = ET.SubElement(par, H_SPAN)
             elem0.text = result.group(1)
             for k, v in elem0.attrib.items():
@@ -555,20 +544,10 @@ class PDFArgs(AbstractArgs):
 
             par.remove(span)
 
-    @property
-    def module_stem(self):
-        """name of module"""
-        return Path(__file__).stem
-
-    # def create_range_list(self):
-    #     """makes list of ranges from pairs on numbers"""
-    #     range_list = range(1,999)
-    #     if type(self.pages) is list:
-    #         range_list = []
-    #         ll = list(map(int, self.pages))
-    #         for i in range(0, len(ll), 2):
-    #             range_list.append(ll[i:i + 2])
-    #     return range_list
+    # @property
+    # def module_stem(self):
+    #     """name of module"""
+    #     return Path(__file__).stem
 
     @classmethod
     def make_page_ranges(cls, raw_page_ranges, offset=0):
@@ -646,7 +625,7 @@ class PDFArgs(AbstractArgs):
         pdf_args.arg_dict[INPATH] = inpath
         assert pdf_args.arg_dict[INPATH].exists(), f"file does not exist {inpath}"
         if chapter_dict is not None:
-            print(f"chapter_dict {chapter_dict}")
+            logger.debug(f"chapter_dict {chapter_dict}")
             maxpage = chapter_dict[chapter]["pages"]
             pdf_args.arg_dict[MAXPAGE] = int(maxpage)
         if outdir is not None:
@@ -654,7 +633,7 @@ class PDFArgs(AbstractArgs):
         pdf_args.arg_dict[OUTDIR] = outdir
         pdf_args.arg_dict[OUTPATH] = Path(outdir, "ipcc_spans.html")
         pdf_args.unwanteds = unwanteds
-        print(f"arg_dict {pdf_args.arg_dict}")
+        logger.debug(f"arg_dict {pdf_args.arg_dict}")
         return pdf_args
 
     def pdf_to_styled_html_CORE(
@@ -697,7 +676,7 @@ class PDFArgs(AbstractArgs):
         styles = CSSStyle.extract_styles_from_html_head_element(html_elem)
         with open(outpath1, "wb") as f:
             f.write(ET.tostring(html_elem, encoding="UTF-8"))
-        print(f"wrote styled html {outpath1}")
+        logger.info(f"wrote styled html {outpath1}")
         style_dict = CSSStyle.create_style_dict_from_styles(style_elems=styles)
         return style_dict
 
@@ -713,20 +692,20 @@ def parse_and_process_1(pdf_args):
         pdf_args.parse_and_process()
         pdf_args.convert_write()
     except Exception as e:
-        print(f"traceback: {traceback.format_exc()}")
-        print(f"******Cannot run pyami******; see output for errors: {e} ")
+        logger.debug(f"traceback: {traceback.format_exc()}")
+        logger.error(f"******Cannot run pyami******; see output for errors: {e} ")
 
 
 def main(argv=None):
     """entry point for PDF conversiom
     typical:
-    python -m pyamihtmlx.ami_pdf \
+    python -m amilib.ami_pdf \
         --inpath /Users/pm286/workspace/pyami/test/resources/ar6/Chapter06/fulltext.pdf \
         --outdir /Users/pm286/workspace/pyami/temp/pdf/chap6/
         --maxpage 100
 
     """
-    print(f"running PDFArgs main")
+    logger.debug(f"running PDFArgs main")
     pdf_args = PDFArgs()
     parse_and_process_1(pdf_args)
 
