@@ -4,6 +4,7 @@ import logging
 import os
 import pprint
 import re
+from io import StringIO
 from pathlib import Path
 from urllib.request import urlopen
 
@@ -13,13 +14,14 @@ import lxml.etree
 import requests
 # import tkinterweb
 from lxml import etree as ET
-from lxml.etree import _Element, _ElementTree
+from lxml.etree import _Element, _ElementTree, _ElementUnicodeResult, XPathEvalError
 from lxml.html import HTMLParser
 # import tkinter as tk
 
 from amilib.file_lib import FileLib
+from amilib.util import Util
 
-logging.debug("loading xml_lib")
+logger = Util.get_logger(__name__)
 
 # make leafnodes and copy remaning content as XML
 TERMINAL_COPY = {
@@ -153,6 +155,8 @@ logging.debug(f"===========LOGGING {logger.level} .. {logging.DEBUG}")
 
 
 class XmlLib:
+    SENTENCE_RE = ".*\\.(\\s*$|\\s+[A-Z].*)" # maybe obsolete
+    SENTENCE_START_RE = ".*\\.\\s+[A-Z].*"
 
     def __init__(self, file=None, section_dir=SECTIONS):
         self.max_file_len = 30
@@ -242,7 +246,7 @@ class XmlLib:
         isect = 0
         for child in children:
             if "ProcessingInstruction" in str(type(child)):
-                # print("PI", child)
+                # logger.debug("PI", child)
                 continue
             if "Comment" in str(type(child)):
                 continue
@@ -271,7 +275,7 @@ class XmlLib:
                     with open(filename1, "wb") as f:
                         f.write(xml_string)
                 except Exception:
-                    print(f"cannot write {filename1}")
+                    logger.debug(f"cannot write {filename1}")
             else:
                 subdir = os.path.join(outdir, filename)
                 # creates empty dirx, may be bad idea
@@ -309,11 +313,16 @@ class XmlLib:
         :param xpaths: """
         xpaths = [xpaths] if type(xpaths) is str else xpaths
         if debug:
-            print(f"xpaths for removal {xpaths}")
+            logger.debug(f"xpaths for removal {xpaths}")
         for xpath in xpaths:
-            elems = elem.xpath(xpath)
+            try:
+                elems = elem.xpath(xpath)
+            except XPathEvalError as e:
+                logger.error(f"bad Xpath: {xpath}")
+                continue
+
             if debug:
-                print(f"elems to remove {elems}")
+                logger.debug(f"elems to remove {elems}")
             for el in elems:
                 if el.getparent() is not None:
                     el.getparent().remove(el)
@@ -348,7 +357,7 @@ class XmlLib:
         """replace nodes with specific text
 
         """
-        print(data, xpath, replacement)
+        logger.debug(data, xpath, replacement)
         tree = ET.fromstring(data)
         for r in tree.xpath(xpath):
             XmlLib.replace_node_with_text(r, replacement)
@@ -356,7 +365,7 @@ class XmlLib:
 
     @classmethod
     def replace_node_with_text(cls, r, replacement):
-        print("r", r, replacement, r.tail)
+        logger.debug("r", r, replacement, r.tail)
         text = replacement
         if r.tail is not None:
             text += r.tail
@@ -389,9 +398,15 @@ class XmlLib:
         :param new_parent: new parent for removed nodes
         :param debug: output debug (def = False)
         """
+        if elem is None or xpath is None:
+            return None
+        if type(xpath) is list:
+            for xp in xpath:
+                cls.remove_elements(elem, xp)
+            return
         elems = elem.xpath(xpath, debug=True)
         if debug:
-            print(f"{xpath} removes {len(elems)} elems")
+            logger.debug(f"{xpath} removes {len(elems)} elems")
         for elem in elems:
             XmlLib.remove_element(elem), debug
             if new_parent is not None:
@@ -419,7 +434,7 @@ class XmlLib:
 
         transform = ET.XSLT(xslt_root)
         if transform.error_log:
-            print("bad xsl? XSLT log", transform.error_log)
+            logger.debug("bad xsl? XSLT log", transform.error_log)
         result_tree = transform(xmlstring)
         assert (result_tree is not None)
         html_root = result_tree.getroot()
@@ -487,6 +502,8 @@ class XmlLib:
         """
         if not path:
             return
+        if elem is None:
+            return
         path = Path(path)
         if mkdir:
             path.parent.mkdir(parents=True, exist_ok=True)
@@ -498,10 +515,11 @@ class XmlLib:
 
                 xmlstr = lxml.etree.tostring(elem).decode(encoding)
             except Exception as e:
-                raise ValueError(f"****** cannot decode XML to {path}: {e} *******")
+                logger.debug(f"****** cannot decode XML to {path}: {e} *******")
+                return None
             try:
                 if debug:
-                    print(f"writing XML {path}")
+                    logger.debug(f"writing XML {path}")
                 f.write(xmlstr)
             except Exception as ee:
                 raise Exception(f"cannot write XMLString {ee}")
@@ -570,10 +588,10 @@ class XmlLib:
         pred_string = f"" if predicate is None else f"{predicate}"
         xp = f"following::*{pred_string}"
         xp = f"following::*"
-        print(f"xp: {xp} {lxml.etree.tostring(elem)}")
+        logger.debug(f"xp: {xp} {lxml.etree.tostring(elem)}")
         nexts = elem.xpath(xp)
         # next = None if len(nexts) == 0 else nexts[:count]
-        print(f"nexts {len(nexts)}")
+        logger.debug(f"nexts {len(nexts)}")
         return nexts
 
     @classmethod
@@ -582,7 +600,7 @@ class XmlLib:
             return None
         parent = elem.getparent()
         if parent is None and debug:
-            print(f" parent of {elem} is None")
+            logger.debug(f" parent of {elem} is None")
         return parent
 
     @classmethod
@@ -593,7 +611,7 @@ class XmlLib:
         """
         if not github_url:
             return None
-        # print(f"url: {github_url}")
+        # logger.debug(f"url: {github_url}")
         if url_cache:
             xml_elem = url_cache.read_xml_element_from_github(github_url)
         else:
@@ -617,7 +635,7 @@ class XmlLib:
         :param debug: print removed elements
         """
         if elem is None:
-            print(f"remove clutter : element is None")
+            logger.debug(f"remove clutter : element is None")
             return
         if declutter is None:
             declutter = DEFAULT_DECLUTTER
@@ -642,7 +660,7 @@ class XmlLib:
 
             if t2 != t1:
                 if debug:
-                    print(f"replaced {t1} by {t2}")
+                    logger.debug(f"replaced {t1} by {t2}")
                 text_elem.text = t2
                 return 1
         return 0
@@ -684,7 +702,7 @@ class XmlLib:
         :param repeat: repeats split on (new) rh span
         :return: None if no match, else first match in span
         """
-        print(f"USE TEMPLATES INSTEAD for HREF or ID generation")
+        logger.debug(f"USE TEMPLATES INSTEAD for HREF or ID generation")
         type_span = type(span)
         parent = span.getparent()
 
@@ -693,23 +711,23 @@ class XmlLib:
             return None
         text = span.text
         if text is None:
-            # print(f"text is None")
+            # logger.debug(f"text is None")
             return None
         if regex is None:
-            print("regex is None")
+            logger.debug("regex is None")
             return None
         match = None
         try:
             match = re.search(regex, text)
         except Exception as e:
-            print(f"bad match {regex} /{e} --> {text}")
+            logger.debug(f"bad match {regex} /{e} --> {text}")
             return
         idx = parent.index(span)
         dummy_templater = Templater()
         # enhanced_regex = EnhancedRegex(regex=regex)
         if match:
             anchor_text = match.group(0)
-            print(f"matched: {regex} {anchor_text}")
+            logger.info(f"matched: {regex} {anchor_text}")
             # href_new = enhanced_regex.get_href(href, text=anchor_text)
             # make 3 new spans
             # some may be empty
@@ -720,11 +738,6 @@ class XmlLib:
                                                                              href=href_new)
             offset += 1
             offset, span2 = cls.create_span(idx, match, offset, parent, span, text, "end")
-            # span_last_text = text[match.span()[1]:]
-            # if len(span_last_text) > 0 :
-            #     span2 = cls.new_span(parent, idx + offset, span, span_last_text)
-            # else:
-            #     print(f"zero-length span2 in {span.text}")
             if ids and type(ids) is str:
                 ids = [None, ids, None]
             if ids and len(ids) == 3:
@@ -743,12 +756,12 @@ class XmlLib:
             if clazz is not None:
                 mid.attrib["class"] = clazz
             if span2 is not None:
-                print(f"style {span2.attrib.get('style')}")
+                logger.debug(f"style {span2.attrib.get('style')}")
 
             parent.remove(span)
             # recurse in RH split
             if regex is None:
-                print("no regex")
+                logger.error("no regex")
                 return
             if repeat > 0:
                 repeat -= 1
@@ -779,7 +792,7 @@ class XmlLib:
             # new_span = XmlLib.create_and_add_anchor(href, span, span_text)
             offset += 1
         else:
-            print(f"zero-length span0 in {span.text}")
+            logger.error(f"zero-length span0 in {span.text}")
             pass
         return offset, new_span
 
@@ -795,14 +808,202 @@ class XmlLib:
         a_elem.text = atext
         span.text = None
 
+    """
+    MIXED CONTENT IN lxml (!!!) NOT EASY
+    see https://lxml.de/tutorial.html#using-xpath-to-find-text
+    and 
+    https://stackoverflow.com/questions/57301789/how-can-i-select-and-update-text-nodes-in-mixed-content-using-lxml
+    
+    the text references the previous elem through getparent()
+
+    >> I found the key to this solution in the docs: Using XPath to find text
+    
+    >> Specifically the is_text and is_tail properties of _ElementUnicodeResult.
+
+    """
+    @classmethod
+    def debug_direct_text_children(cls, elem):
+        """
+        iterate over sequence of elements and texts and replace by span(text)
+        This gets rid of the node + node.tail  and converts the texts to a first class element
+        :param elem: element with child elements and child texts
+        Assumes no PIs, no textdata
+        """
+        if elem is None:
+            return
+        children = elem.xpath("*|text()")
+        for child in children:
+            if type(child) is _ElementUnicodeResult:
+                 logger.info(f"T:{child}:Tail:{child.is_tail}, Text:{child.is_text}, Parent:{child.getparent().tag}")
+            elif type(child) is _Element:
+                logger.debug(f"E:{child}|{child.tail}")
+            else:
+                logger.debug(f"?:{child}")
+
+    @classmethod
+    def replace_tail_text_with_span(cls, elem):
+        """
+        <foo>...</foo>tail
+        is replaced by
+        <foo>...</foo><span>tail<span>
+        :param elem: to be edited
+        """
+        if elem is None or elem.getparent() is None:
+            return
+        if elem.tail is not None:
+            parent = elem.getparent()
+            span = ET.Element("span")
+            span.text = elem.tail
+            elem.tail = None
+            elem.addnext(span)
+
+    @classmethod
+    def replace_child_tail_texts_with_spans(cls, parent_elem):
+        """
+        find all (tail) text childrren and wrap in spans
+        :param parent_elem: parent elem with child texts
+        """
+        if parent_elem is None:
+            return
+        texts = parent_elem.xpath("./text()")
+        for text in texts:
+            pre_elem = text.getparent()
+            cls.replace_tail_text_with_span(pre_elem)
+        return
+
+    def get_sentence_breaks(self, texts):
+        """
+        split html/text iinto sentences
+        :param texts: mixed content texts (tails)
+        :return: array of splits (still being developed)
+        """
+        # texts = self.get_texts()
+
+        text_breaks = [(j, t) for (j, t) in enumerate(texts) if re.match(self.SENTENCE_RE, t)]
+        splits = []
+        for (idx, txt) in text_breaks:
+            prec = '^' if idx == 0 else texts[idx - 1]
+            foll = '$' if idx == len(texts) - 1 else texts[idx + 1]
+            splits.append(f"|{prec}|{txt}|{foll}|")
+        return splits
+
+    @classmethod
+    def add_sentence_brs(cls, texts):
+        """
+        split html/text in paragrah into sentences
+        add <br> element after period
+        :return: None (modifies texts) (still being developed)
+        """
+        if len(texts) == 0:
+            return
+        parent = texts[0].getparent().getparent()
+        logger.debug(f"parent: {parent}")
+        for text in texts:
+            head = text.getparent()
+            # logger.debug(f"head: {head.tag}")
+            if re.match(cls.SENTENCE_START_RE, text):
+                splits = text.split('.', 1)
+                prec_elem = text.getparent()
+                prec_elem.tail = splits[0] + "."
+                br_elem = ET.Element("br")
+                br_elem.tail = splits[1]
+                prec_elem.addnext(br_elem)
+        return
+
+    @classmethod
+    def element_to_string(cls, element, method="xml", pretty_print=True, encoding="UTF-8"):
+        """
+        converts element to string
+        :param element: to render
+        :param method: "html" or "xml",default "xml"
+        :param pretty_print: pretty print , default True
+        :param encoding: default UTF-8
+
+        """
+        return None if element is None else lxml.etree.tostring(
+            element, method=method, pretty_print=pretty_print).decode(encoding)
+
+    @classmethod
+    def get_single_element(cls, xmlx, xpath):
+        """
+        Convenience method to avoid testing for len()
+        gets a single element from xpath or returns None
+        :param xmlx: lxml element to query
+        :param xpath:
+        :return: single element or None
+        """
+        if xmlx is None:
+            logger.debug("xmlx is None")
+            return None
+        if xpath is None:
+            logger.debug("xpath is None ")
+            return None
+        results = xmlx.xpath(xpath)
+        if len(results) == 1:
+            return results[0]
+        else:
+            return None
+
 
 class HtmlElement:
     """to provide fluent HTML builder and parser NYI"""
     pass
 
 
+class HtmlEditor:
+    """
+    Convenience method for creating HTML tree
+    skeleton html with attributes for head, style, body
+    hardcoded attributes (Html elements) are:
+    .html - the whole document
+    .head - the head
+    .style - a style stub in head (add others with SubElement)
+    .body - a stub body
+    (To create other elemnts you have to use SubElement or append)
+
+    example:
+    skel = HtmlSkeleton()
+    skel.style.text = "p {background: pink;}"
+    p = ET.SubElement(skel.body, "p")
+    p.text = "foo"
+    p = ET.SubElement(skel.body, "p")
+    p.text = "bar"
+    skel.write(myfile, debug=True)
+
+
+
+    """
+
+    def __init__(self):
+        """
+        creates elements self.html, self.head, self.body
+        """
+        self.html = lxml.etree.Element("html")
+        self.head = ET.SubElement(self.html, "head")
+        self.body = ET.SubElement(self.html, "body")
+
+    def write(self, file, debug=True):
+        HtmlLib.write_html_file(self.html, file, debug=debug)
+
+    def add_style(self, selector, value):
+        """
+        at present just use HTML style content,
+        content includes the "{...}'
+        e.g.
+        selector ="span"
+        value = "{background, pink; border: solid 1px blue;}
+        htmlx.add_style(selector, value)
+
+        """
+
+        style = ET.SubElement(self.head, "style")
+        style.text = f"{selector} {value}"
+
+
 class HtmlLib:
 
+    CLASS_ATTNAME = "class"
+
     @classmethod
     def convert_character_entities_in_lxml_element_to_unicode_string(cls, element, encoding="UTF-8") -> str:
         """
@@ -833,6 +1034,20 @@ class HtmlLib:
         html_elem.append(lxml.etree.Element("body"))
         return html_elem
 
+    def create_html_container_with_head_style_body(cls):
+        """
+        creates
+        <html>
+          <head/>
+          <body/>
+        </html>
+        """
+        html_container = Html_Container()
+        html_elem = lxml.etree.Element("html")
+        html_elem.append(lxml.etree.Element("head"))
+        html_elem.append(lxml.etree.Element("body"))
+        return html_elem
+
     @classmethod
     def add_copies_to_head(cls, html_elem, elems):
         """copies elems and adds them to <head> of html_elem
@@ -867,15 +1082,15 @@ class HtmlLib:
         """
         if html_elem is None:
             html_elem = HtmlLib.create_html_with_empty_head_body()
-        heads = html_elem.xpath("/html/head")
-        return heads[0] if len(heads) == 1 else None
+        head = XmlLib.get_single_element(html_elem, "/html/head")
+        return head
 
     @classmethod
     def add_base_url(cls, html_elem, base_url):
         head = cls.get_head(html_elem)
         base = head.xpath("base")
         if len(base) > 1:
-            print(f"too many base_urls; probable error")
+            logger.info(f"too many base_urls; probable error")
             return
         if len(base) == 0:
             base = lxml.etree.SubElement(head, "base")
@@ -935,11 +1150,11 @@ class HtmlLib:
         """
         if html_elem is None:
             if debug:
-                print("null html elem to write")
+                logger.info("null html elem to write")
             return
         if outfile is None:
             if debug:
-                print("no outfile given")
+                logger.error("no outfile given")
             return
         if type(html_elem) is _ElementTree:
             html_elem = html_elem.getroot()
@@ -948,7 +1163,7 @@ class HtmlLib:
         if encoding and encoding.lower() == "utf-8":
             head = HtmlLib.get_or_create_head(html_elem)
             if head is None:
-                print(f"cannot create <head> on html elem; not written")
+                logger.error(f"cannot create <head> on html elem; not written")
                 return
 
         outdir = os.path.dirname(outfile)
@@ -956,8 +1171,7 @@ class HtmlLib:
             Path(outdir).mkdir(exist_ok=True, parents=True)
 
         # cannot get this to output pretty_printed, (nor the encoding)
-        tobytes = lxml.etree.tostring(html_elem, method="html", pretty_print=pretty_print)
-        tostring = tobytes.decode("UTF-8")
+        tostring = lxml.etree.tostring(html_elem, method="html", pretty_print=pretty_print).decode("UTF-8")
 
         with open(str(outfile), "w") as f:
             f.write(tostring)
@@ -978,7 +1192,7 @@ class HtmlLib:
         if html_elem is None:
             return None
         if html_elem.tag.lower() != "html":
-            print(f"not a full html element")
+            logger.error(f"not a full html element")
             return None
         head = HtmlLib.get_head(html_elem)
         if head is None:
@@ -991,7 +1205,7 @@ class HtmlLib:
         """adds <meta charset=charset" to <head>"""
         head = HtmlLib.get_or_create_head(html_elem)
         if head is None:
-            print(f"cannot create <head>")
+            logger.error(f"cannot create <head>")
             return
         cls.remove_charsets(head)
         meta = lxml.etree.SubElement(head, "meta")
@@ -1017,7 +1231,7 @@ class HtmlLib:
                 matchstr = regex.match(span.text)
                 if matchstr:
                     if debug:
-                        print(f"matched {matchstr.group(1)} {span.text[:50]}")
+                        logger.info(f"matched {matchstr.group(1)} {span.text[:50]}")
                     sectionlist.append(span)
         return sectionlist
 
@@ -1025,23 +1239,42 @@ class HtmlLib:
     def parse_html(cls, infile):
         """
         parse html file as checks for file existence
-        :param infile: file to parse
+        :param infile: file to parse or url (checks prefix)
         :return: root element
         """
         if not infile:
-            print(f"infile is None")
+            logger.error(f"infile is None")
             return None
-        path = Path(infile)
-        if not path.exists():
-            print(f"file does not exist {infile}")
-            return None
-        else:
-            try:
-                html_tree = lxml.etree.parse(str(infile))
-                return html_tree.getroot()
-            except Exception as e:
-                print(f"cannot parse {infile} because {e}")
+        if not str(infile).startswith("http"):
+            path = Path(infile)
+            if not path.exists():
+                logger.error(f"file does not exist {infile}")
                 return None
+        try:
+            infile = "https://en.wikipedia.org"
+            logger.debug(f"infile {infile}")
+            html_tree = lxml.html.parse(infile, HTMLParser())
+            if html_tree is None:
+                logger.error(f"Cannot parse {infile}, returned None")
+            return html_tree.getroot()
+        except Exception as e:
+            logger.error(f"cannot parse {infile} because {e}")
+            return None
+
+    @classmethod
+    def parse_html_string(cls, string):
+        """
+        parse string
+        :param string: html
+        :return: html element or None
+        """
+        try:
+            html_element = lxml.html.fromstring(string)
+        except Exception as e:
+            logger.error(f"html error {e}")
+            return None
+
+        return html_element
 
     @classmethod
     def find_paras_with_ids(cls, html, xpath=None):
@@ -1059,21 +1292,118 @@ class HtmlLib:
         return paras
 
     @classmethod
-    def para_contains_phrase(cls, para, phrase, ignore_case=True):
+    def para_contains_phrase(cls, para, phrase, ignore_case=True, markup=None):
+        """
+        search paragraph with phrase. If markuip is not None add hyperlinks
+
+        Parameters
+        ----------
+        para paragraph to search
+        phrase search phrase
+        ignore_case if True lowercase text and phrase
+        markup if True search each itertext and insert hrefs, else just seatch concatenation
+
+        Returns
+        -------
+
+        """
         if ignore_case:
             phrase = phrase.lower()
-        text = "".join(para.itertext())
-        if ignore_case:
-            text = text.lower()
-        if re.search(r'\b' + phrase + r'\b', text):
-            return True
+        search_re = r'\b' + phrase + r'\b'
+        if not markup:
+            text = "".join(para.itertext())
+            if ignore_case:
+                text = text.lower()
+            if re.search(search_re, text):
+                return True
+        else:
+            texts = para.xpath(".//text()")
+            for text in texts:
+                match = re.search(search_re, text)
+                if match:
+                    cls._insert_ahref(markup, match, phrase, text)
+
         return False
 
     @classmethod
-    def create_para_ohrase_dict(cls, paras, phrases):
+    def _insert_ahref(cls, url_base, match, phrase, text):
+        """
+        Add hyperlinks to text. The order of opratyations matters
+        """
+        id = HtmlLib.generate_id(phrase)
+        href, title = cls._create_href_and_title(id, url_base)
+
+        # text before, inside and after <a> element
+        start_ = text[0:match.start()]
+        mid_ = text[match.start():match.end()]
+        end_ = text[match.end():]
+
+        # might be a text (contained within lead) or tail following it
+
+        # text contained in element
+        if text.is_text:
+            aelem = cls.add_href_for_lxml_text(start_, text)
+
+        # text following element
+        elif text.is_tail:
+            aelem = cls._add_href_for_lxml_tail(start_, text)
+        else:
+            logger.error(f"ERROR??? (not text of tail) {start_}|{mid_}|{end_}")
+
+        # add content and attributes to aelem
+        aelem.attrib["href"] = href
+        aelem.text = mid_
+        aelem.tail = end_
+        if title:
+            aelem.attrib["title"] = title
+
+    @classmethod
+    def _add_href_for_lxml_tail(cls, start_, text):
+        prev = text.getparent()
+        aelem = ET.Element("a")
+        aelem.attrib["style"] = "border:solid 1px; background: #ffbbbb;"
+        prev.addnext(aelem)  # order metters1
+        prev.tail = start_ + " "
+        return aelem
+
+    @classmethod
+    def add_href_for_lxml_text(cls, start_, text):
+        parent = text.getparent()
+        tail = parent.tail
+        aelem = ET.SubElement(parent, "a")
+        aelem.attrib["style"] = "border:solid 1px; background: #ffffbb;"
+        parent.text = start_
+        parent.tail = tail
+        return aelem
+
+    @classmethod
+    def _create_href_and_title(cls, id, url_base):
+        href = f"{url_base}"
+        href_elem = ET.parse(href, HTMLParser())
+        idelems = href_elem.xpath(f".//*[@id='{id}']")
+        title = id
+        if len(idelems) > 0:
+            ps = idelems[0].xpath("./p")
+            if len(ps) > 0:
+                p = ps[0] if len(ps) == 1 else ps[1]
+                if p is not None:
+                    title = "".join(p.itertext())
+        return href, title
+
+    @classmethod
+    def generate_id(cls, phrase):
+        """
+        strip, converts whitespace to single "-" and lowercase
+        """
+        phrase1 = re.sub(r"\s+", "_", phrase)
+        return phrase1
+
+    @classmethod
+    def search_phrases_in_paragraphs(cls, paras, phrases, markup=None):
         """search for phrases in paragraphs
         :param paras: list of HTML elems with text (normally <p>), must have @id else ignored
         :param phrases: list of strings to search for (word boundary honoured)
+        :param markup:html dictionary with phrases
         :return: dict() keyed on para_ids values are dict of search hits by phrase
         """
         para_phrase_dict = dict()
@@ -1083,7 +1413,7 @@ class HtmlLib:
                 continue
             phrase_dict = dict()
             for phrase in phrases:
-                count = HtmlLib.para_contains_phrase(para, phrase, ignore_case=True)
+                count = HtmlLib.para_contains_phrase(para, phrase, ignore_case=True, markup=markup)
                 if count > 0:
                     phrase_dict[phrase] = count
                     para_phrase_dict[para_id] = phrase_dict
@@ -1103,6 +1433,64 @@ class HtmlLib:
 
         return html
 
+    @classmethod
+    def _extract_paras_with_ids(cls, infile, count=-1):
+        """
+
+        Parameters
+        ----------
+        infile html file with p[@id]
+        count number of paragraphs with @id (default -1) . if count >= 0, asserts number flound == count
+
+        Returns
+        -------
+
+        """
+        assert Path(infile).exists(), f"{infile} does not exist"
+        html = ET.parse(str(infile), HTMLParser())
+        paras = HtmlLib.find_paras_with_ids(html)
+        if count >= 0:
+            assert len(paras) == count
+        return paras
+
+    @classmethod
+    def add_link_stylesheet(self, css_file, htmlx):
+        """
+        add stylesheet link to <head>
+        creates <link rel='stylesheet' href=<css_file/>
+        does not remove other links
+        :param htmlx: html element with existing <head> element
+        :param css_file:file path , absolute or relative to html file
+
+        """
+
+        link = ET.SubElement(HtmlLib.get_head(htmlx), "link")
+        link.attrib["rel"] = "stylesheet"
+        link.attrib["href"] = css_file
+
+    @classmethod
+    def add_base_to_head(self, htmlx, base_href):
+        """
+        create or reuse a single <base> child of <head> and add self.base as href value
+        :param: html root element
+        :param base_href: value for @href on <base>
+        """
+        if htmlx is None:
+            logger.error("no HTML element")
+            return
+        if base_href is None:
+            logger.error("no base_href")
+            return
+        head = HtmlLib.get_head(htmlx)
+        bases = head.xpath("./base")
+        if len(bases) == 1:
+            base = bases[0]
+        else:
+            # create base
+            base = ET.SubElement(HtmlLib.get_head(htmlx), "base")
+        base.attrib["href"] = base_href
+
+
 
 class DataTable:
     """
@@ -1134,7 +1522,7 @@ class DataTable:
         self.add_rows(rowdata)
         self.head = None
         self.title = None
-        self.title.text = None
+        # self.title.text = None
 
     def create_head(self, title):
         """
@@ -1220,7 +1608,6 @@ class DataTable:
             for val in row:
                 td = ET.SubElement(tr, H_TD)
                 td.text = val
-                # print("td", td.text)
 
     def make_row(self):
         """
@@ -1249,15 +1636,11 @@ class DataTable:
         with open(data_table_file, "w") as f:
             text = bytes.decode(ET.tostring(self.html))
             f.write(text)
-            print("WROTE", data_table_file)
+            logger.info("WROTE", data_table_file)
 
     def __str__(self):
-        # s = self.html.text
-        # print("s", s)
-        # return s
-        # ic("ichtml", self.html)
         htmltext = ET.tostring(self.html)
-        print("SELF", htmltext)
+        logger.info("SELF", htmltext)
         return htmltext
 
 
@@ -1289,7 +1672,7 @@ class Templater:
         elif template_type == ID_TEMPLATE:
             template = self.id_template
         else:
-            print(f"***Bad template type** {template_type}")
+            logger.error(f"***Bad template type** {template_type}")
             return None
         return Templater.get_matched_template(self.regex, strng, self.template)
 
@@ -1325,7 +1708,7 @@ class Templater:
         but more complex templates can include repeats. However these are NOT f-strings and do not use eval()
         """
         if regex is None:
-            print(f"**************regex is None")
+            logger.error(f"**************regex is None")
             return None
         if template is None:
             raise ValueError("template shuuld not be None")
@@ -1343,7 +1726,7 @@ class Templater:
     def create_template(cls, template=None, regex=None, href_template=None, id_template=None):
         templater = Templater()
         if not regex:
-            print(f"no regex in templater")
+            logger.error(f"no regex in templater")
             return None
         templater.regex = regex
         templater.template = template
@@ -1367,7 +1750,7 @@ class Templater:
         :return: None if no match, else first match in span
         """
         if span is None:
-            print(f"span is None")
+            logger.error(f"span is None")
             return None
         type_span = type(span)
         parent = span.getparent()
@@ -1381,19 +1764,19 @@ class Templater:
         match = None
         regex = self.regex
         if regex is None:
-            print(f"************no regex in templater")
+            logger.error(f"************no regex in templater")
             return
         try:
             match = re.search(regex, text)
         except Exception as e:
-            print(f"bad match {regex} /{e} => {text}")
+            logger.error(f"bad match {regex} /{e} => {text}")
             return
         idx = parent.index(span)
         # enhanced_regex = EnhancedRegex(regex=regex)
         if match:
             anchor_text = match.group(0)
             if debug:
-                print(f"matched: {regex} {anchor_text}")
+                logger.info(f"matched: {regex} {anchor_text}")
             # href_new = enhanced_regex.get_href(href, text=anchor_text)
             # make 3 new spans
             # some may be empty
@@ -1427,7 +1810,6 @@ class Templater:
 
         if self.href_template:
             href = self.match_href_template(textx)
-            # print(f">>>>  {href}..{self}")
             XmlLib.create_and_add_anchor(href, new_span, textx)
         elif href:
             XmlLib.create_and_add_anchor(href, new_span, textx)
@@ -1469,7 +1851,7 @@ class Templater:
         for template_ref in template_ref_list:
             sub_markup_dict = markup_dict.get(template_ref)
             if not sub_markup_dict:
-                print(f"cannot find template {template_ref}")
+                logger.error(f"cannot find template {template_ref}")
                 continue
             regex = sub_markup_dict.get("regex")
             target_template = sub_markup_dict.get("target_template")
@@ -1493,14 +1875,13 @@ class Templater:
         """
         divs = html_elem.xpath(id_xpath)
         if len(divs) == 0:
-            print(f"cannot find id {id_xpath}")
+            logger.error(f"cannot find id {id_xpath}")
             return
         div = divs[0]
         div_content = ''.join(html_elem.itertext())
-        # print(f" div_content {div_content[:maxchar]}")
         templater = Templater.create_template(template, regex)
         id = templater.match_template(div_content, template_type=ID_TEMPLATE)
-        print(f">>id {id}")
+        logger.debug(f">>id {id}")
         return id
 
 
@@ -1535,7 +1916,7 @@ def main():
 
 def test_xml():
     xml_string = "<a>foo <b>and</b> with <d/> bar</a>"
-    print(XmlLib.remove_all_tags(xml_string))
+    logger.debug(XmlLib.remove_all_tags(xml_string))
 
 
 def test_data_table():
@@ -1563,7 +1944,7 @@ def test_replace_strings_with_unknown_encodings():
     target = "âaerosol particlesâ."
     assert len(tuple_list[0][0]) == 3
     sout = XmlLib.iteratively_replace_strings(tuple_list, target)
-    print(sout)
+    logger.debug(sout)
     assert sout == "‘aerosol particles’."
 
 
