@@ -9,6 +9,12 @@ from pathlib import Path
 
 from lxml import etree
 
+# problems with Python loader
+import idna
+import certifi
+import SPARQLWrapper
+
+from amilib.file_lib import FileLib
 # local
 from amilib.util import Util
 from amilib.wikimedia import WikidataSparql as WS
@@ -22,6 +28,8 @@ skip_config_test = True
 # TODO needs local config file
 # @unittest.skipIf(skip_config_test, "needs local config")
 
+logger = Util.get_logger(__name__)
+
 class AmiAnyTest(unittest.TestCase):
     # for marking and skipping unittests
     # skipUnless
@@ -37,6 +45,8 @@ class AmiAnyTest(unittest.TestCase):
     USER = True   # user-facing test
     BUG = True    # skip BUGs
 
+    PMR_NAME ="pm286"
+    IS_PMR = (PMR_NAME == Util.get_username())
     # outputs for tests
 
     # temporary output data (can be deleted after tests)
@@ -69,9 +79,6 @@ class AmiAnyTest(unittest.TestCase):
         pass
 
     def tearDown(self) -> None:
-        # print(f"argv_copy {self.argv_copy}")
-        # print(f"argv {sys.argv}")
-        # self.argv = list(self.argv_copy)
         pass
 
     # used to control long tests. Crude but robust (other markers are more complex)
