@@ -1,17 +1,14 @@
 import csv
-import glob
-import re
 import unittest
-from collections import Counter
 from pathlib import Path
 
 import lxml.etree as ET
-from geopy.geocoders import Nominatim
 from lxml.html import HTMLParser, Element, HtmlElement
 
 from amilib.amidriver import AmiDriver, URL, XPATH, OUTFILE  # EXPAND_SECTION_PARAS
 from amilib.file_lib import FileLib
 from amilib.headless_lib import HeadlessLib
+from amilib.util import Util
 from amilib.wikimedia import WikidataLookup, WikidataPage
 from amilib.xml_lib import XmlLib, HtmlLib, DECLUTTER_BASIC
 from test.resources import Resources
@@ -49,46 +46,13 @@ EXPAND_SECTION_PARAS = [
 
 
 
-class MiscTest(AmiAnyTest):
-
-    @unittest.skip("requires connection and output can micro-vary")
-    def test_geolocate_GEO(self):
-        """
-        GEO: locates places by name using Nominatim
-
-        (Test occasionally returns variable decimal places
-        TODO needs a fixed-place numeric comparison
-        :return:
-        """
-        geolocator = Nominatim(timeout=10, user_agent="semanticclimate@gmail.com")
-        results = []
-        for name in [
-            "Benares",
-#            "Bengaluru",
-            "Delhi",
-            "Ladakh",
-            # "Mumbai",
-            "Mysore",
-        ]:
-            location = geolocator.geocode(name)
-            tuple = (name, location[1], location.latitude, location.longitude)
-            results.append(tuple)
-        assert results == [
-            ('Benares', (25.3356491, 83.0076292), 25.3356491, 83.0076292),
-            # ('Bengaluru', (12.9767936, 77.590082), 12.9767936, 77.590082),
-            ('Delhi', (28.6273928, 77.1716954), 28.6273928, 77.1716954),
-            ('Ladakh', (33.9456407, 77.6568576), 33.9456407, 77.6568576),
-            # ('Mumbai', (18.9733536, 72.82810491917377), 18.9733536, 72.82810491917377), # Mumbai seems to move!
-            ('Mysore', (12.3051828, 76.6553609), 12.3051828, 76.6553609),
-        ]
-
-
-
 run_test = False
 force = False
 
+logger = Util.get_logger(__name__)
 
 # force = True # uncomment to run tests with this keyword
+@unittest.skipUnless(AmiAnyTest.IS_PMR, "headless browsing still giving intermittent problems that PMR has to solve")
 class DriverTest(AmiAnyTest):
     """ Currently 8 minutes"""
     """
@@ -108,10 +72,10 @@ class DriverTest(AmiAnyTest):
 
         html_out = Path(SC_TEST_DIR, f"complete_text_{level}.html")
         driver.download_expand_save(url, click_list, html_out, level=level)
-        print(f"elem {driver.get_lxml_element_count()}")
+        logger.debug(f"elem {driver.get_lxml_element_count()}")
         XmlLib.remove_common_clutter(driver.lxml_root_elem)
 
-        print(f"elem {driver.get_lxml_element_count()}")
+        logger.debug(f"elem {driver.get_lxml_element_count()}")
         driver.write_html(Path(html_out))
         driver.quit()
 
@@ -120,6 +84,10 @@ class DriverTest(AmiAnyTest):
         """
         A potential multiclick download
         """
+        """
+        fails with:
+         OSError: [Errno 8] Exec format error:
+         """
         url = SYR_URL + "annexes-and-index/"
         driver = AmiDriver(sleep=SLEEP)
         click_list = EXPAND_SECTION_PARAS
@@ -153,7 +121,7 @@ class DriverTest(AmiAnyTest):
             if len(doc) == 3:
                 url = url + doc[2] + "/"
             url = url + "glossary" + "/"
-            print(f"url: {url}")
+            logger.debug(f"url: {url}")
             GLOSSARY_TOP = "glossary"
             rep_dict = {
                 GLOSSARY_TOP:
@@ -272,9 +240,9 @@ class DriverTest(AmiAnyTest):
         """
         CHAP_PREF = "Chapter"
         for wg in range(3, 4):
-            print(f"wg = {wg}")
+            logger.debug(f"wg = {wg}")
             wg_url = AR6_URL + f"wg{wg}/"
-            print(f"downloading from {wg_url}")
+            logger.info(f"downloading from {wg_url}")
             for ch in range(1, 18):
                 chs = str(ch)
                 if len(chs) == 1:
@@ -319,7 +287,7 @@ class DriverTest(AmiAnyTest):
         GLOSS_INPUT_DIR = Path(TOTAL_GLOSS_DIR, "input")
         assert GLOSS_INPUT_DIR.exists()
         dict_files = sorted(FileLib.posix_glob(f"{GLOSS_INPUT_DIR}/*.html"))
-        print(f"making glossary from {len(dict_files)} files in {GLOSS_INPUT_DIR}")
+        logger.info(f"making glossary from {len(dict_files)} files in {GLOSS_INPUT_DIR}")
         out_dir = Path(TOTAL_GLOSS_DIR, "output")
         HeadlessLib.make_glossary(dict_files, out_dir, total_glossary=TOTAL_GLOSS_DIR, debug=True)
 
@@ -338,11 +306,11 @@ class DriverTest(AmiAnyTest):
         for encoding in encodings_to_try:
             try:
                 decoded_text = text.encode(encoding).decode('utf-8')
-                print(f"Decoded with {encoding}: {decoded_text}")
+                logger.info(f"Decoded with {encoding}: {decoded_text}")
             except UnicodeDecodeError as e1:
-                print(f"Failed to decode with {encoding} goves {e1}")
+                logger.debug(f"Failed to decode with {encoding} goves {e1}")
             except UnicodeEncodeError as e2:
-                print(f"failed encode with {encoding} gives {e2}")
+                logger.warning(f"failed encode with {encoding} gives {e2}")
 
     def test_glossary_encoding_CHAR(self):
         """
@@ -351,7 +319,7 @@ class DriverTest(AmiAnyTest):
         input = Path(TOTAL_GLOSS_DIR, "input", "Adaptation_limits_A.html")
         with open(str(input), "r", encoding="UTF-8") as f:
             content = f.read()
-            print(f"content {content}")
+            logger.debug(f"content {content}")
 
     def test_make_input_output_table(self):
         """
@@ -374,7 +342,7 @@ class DriverTest(AmiAnyTest):
             # make output filename from input name
             output_file = Path(output_dir, str(input_file.stem)[:-2] + ".html")  # strip letter
             if not output_file.exists():
-                print(f"cannot read {output_file}")
+                logger.warning(f"cannot read {output_file}")
                 continue
             output_name = output_file.name
             tr = ET.SubElement(table, "tr")
@@ -400,7 +368,7 @@ class DriverTest(AmiAnyTest):
             assert glossary_file.exists(), f"file should exist {glossary_file}"
             gloss_html = ET.parse(str(glossary_file))
             elements = gloss_html.xpath("//*")
-            print(f"elements {len(elements)}")
+            logger.debug(f"elements {len(elements)}")
 
     def test_wikimedia_WIKI(self):
         """
@@ -410,10 +378,10 @@ class DriverTest(AmiAnyTest):
         entries = total_html.xpath(".//div/h4")
         start = 190
         end = 200
-        print(f"downloading {start} - {end} from {len(entries)} entries")
+        logger.info(f"downloading {start} - {end} from {len(entries)} entries")
         csvfile = Path(TOTAL_GLOSS_DIR, "wiki", f"{start}_{end}.csv")
         csvfile.parent.mkdir(parents=True, exist_ok=True)
-        print(f"writing to {csvfile}")
+        logger.info(f"writing to {csvfile}")
         with open(csvfile, "w") as f:
             wikiwriter = csv.writer(f, quoting=csv.QUOTE_ALL)
             wikiwriter.writerow(["term", "highestQid", "highest_desc", "list_of_others"])
@@ -421,10 +389,10 @@ class DriverTest(AmiAnyTest):
                 if i < start or i > end:
                     continue
                 term = entry.text
-                print(f"entry: {i}; term {term}")
+                logger.info(f"entry: {i}; term {term}")
                 wikidata_lookup = WikidataLookup()
                 qitem0, desc, wikidata_hits = wikidata_lookup.lookup_wikidata(term)
-                print(f"qitem {qitem0, desc}")
+                logger.debug(f"qitem {qitem0, desc}")
                 wikiwriter.writerow([term, qitem0, desc, wikidata_hits])
 
     @unittest.skip("no idea what this does - I only wrote it!")
@@ -434,7 +402,7 @@ class DriverTest(AmiAnyTest):
         TODO move elsewhere
         """
         abbrev_file = Path(TOTAL_GLOSS_DIR, "glossaries", "total", "acronyms.csv")
-        print(f"looking up acronym file {abbrev_file} in Wikidata")
+        logger.info(f"looking up acronym file {abbrev_file} in Wikidata")
         offset = 1000
         count = 0
         MAXCOUNT = 3
@@ -452,10 +420,10 @@ class DriverTest(AmiAnyTest):
                         term = row[1]
                         qitem0, desc, hits = lookup.lookup_wikidata(term)
                         if qitem0 is None:
-                            print(f"failed on text: {row}")
+                            logger.warning(f"failed on text: {row}")
                             # qitem0, desc, hits = lookup.lookup_wikidata(abb)
                             if qitem0 is None:
-                                print(f"failed on text {term} and abbreviation: {abb}")
+                                logger.warning(f"failed on text {term} and abbreviation: {abb}")
                                 out_row = [abb, term, "?", "?", "?"]
                             else:
                                 out_row = [abb, term, qitem0, desc, hits]
@@ -471,7 +439,7 @@ class DriverTest(AmiAnyTest):
         glossdir.mkdir(exist_ok=True, parents=True)
         abbrev_file = Path(glossdir, "acronyms_wiki.csv")
         output_file = Path(glossdir, "acronyms_wiki_pedia.csv")
-        maxout = 20  # 1700 in total
+        maxout = 5  # 1700 in total
         lookup = WikidataLookup()
         with open(output_file, "w") as fout:
             csvwriter = csv.writer(fout)
@@ -488,11 +456,11 @@ class DriverTest(AmiAnyTest):
                     desc = row[3]
                     hits = row[4]
                     if qid is None or not qid.startswith("Q"):
-                        print(f"no QID")
+                        logger.info(f"no QID")
                         continue
                     wikidata_page = WikidataPage(qid)
                     wikipedia_links = wikidata_page.get_wikipedia_page_links(["en"])
-                    print(f"wikipedia links {wikipedia_links}")
+                    logger.info(f"wikipedia links {wikipedia_links}")
                     out_row = [abb, term, qid, desc, hits, wikipedia_links]
                     csvwriter.writerow(out_row)
 
@@ -525,7 +493,7 @@ class DriverTest(AmiAnyTest):
 # ipcc_graph_with_coloured_nodes.to_csv('coloured.csv')
 # make_graph(ipcc_graph_with_coloured_nodes, source='source', target='target', colour ='node_colour')
 
-class TestUtils1:
+class Utils1Test:
 
     @unittest.skip("not written")
     def test_strip_guillemets(self):
