@@ -20,10 +20,9 @@ from test.test_all import AmiAnyTest
 # local
 
 
-logger = logging.getLogger("py4ami_test_util")
+logger = Util.get_logger(__name__)
 
-
-class TestUtil(AmiAnyTest):
+class Util0Test(AmiAnyTest):
     # def __init__(self):
     sys_argv_save = None
 
@@ -207,8 +206,15 @@ class TestUtil(AmiAnyTest):
                                             ['', '(?P<decision>\\d+)', '/', '(?P<type>CP|CMA|CMP)', '\\.',
                                              '(?P<session>\\d+)', ''])
 
+    def test_get_username(self):
+        """gets username
+        """
+        username = Util.get_username()
+        if AmiAnyTest.IS_PMR:
+            assert username == "pm286"
 
-class TestGithubDownloader(AmiAnyTest):
+
+class GithubDownloaderTest(AmiAnyTest):
     # def __init__(self):
     #     pass
 
@@ -219,7 +225,7 @@ class TestGithubDownloader(AmiAnyTest):
         downloader = GithubDownloader(owner=owner, repo=repo, max_level=1)
         page = None
         downloader.make_get_main_url()
-        print(f"main page {downloader.main_url}")
+        logger.info(f"main page {downloader.main_url}")
         url = downloader.main_url
         if not url:
             print(f"no page {owner}/{repo}")
@@ -228,7 +234,7 @@ class TestGithubDownloader(AmiAnyTest):
         downloader.load_page(url, level=0)
 
 
-class TestAmiArgParser(AmiAnyTest):
+class AmiArgParserTest(AmiAnyTest):
 
     def test_ami_arg_parse(self):
         """
@@ -240,7 +246,7 @@ class TestAmiArgParser(AmiAnyTest):
 
         # this works
         arg_dict = ami_argparse.parse_args(["--flt", "3.2"])
-        print(f"arg_dict1 {arg_dict}")
+        logger.debug(f"arg_dict1 {arg_dict}")
         # this fails
         try:
             arg_dict = ami_argparse.parse_args(["--flt", "3.2", "str"])
@@ -250,10 +256,10 @@ class TestAmiArgParser(AmiAnyTest):
             print(f" error {e}")
 
         arg_dict = ami_argparse.parse_args(["--flt", "99.2"])
-        print(f"arg_dict2 {arg_dict}")
+        logger.debug(f"arg_dict2 {arg_dict}")
 
 
-class TestTemplate(AmiAnyTest):
+class TemplateTest(AmiAnyTest):
 
     def test_id_templates(self):
         """Splits files at Decisions"""
@@ -272,13 +278,35 @@ class TestTemplate(AmiAnyTest):
         matched_templates = Templater.get_matched_templates(regex, ss, template)
         assert matched_templates == ['Decision_12_CMP_5', 'Decision_10_CP_17', 'Decision_2_CMA_2', None]
 
-class UtilTests:
-    def test_dict_read(self):
-        file = "section_templates.json"
-        return Util.read_pydict_from_json(file)
+# class UtilTests:
+#     def test_dict_read(self):
+#         file = "section_templates.json"
+#         return Util.read_pydict_from_json(file)
 
-class TestCommandline:
+class CommandlineTest:
 
     def test_command(self):
         command = "--help"
 
+class LoggingTest(AmiAnyTest):
+    import logging
+
+    def test_logger(self):
+        logger = Util.get_logger(__name__)
+
+        logger.info("Server started listening on port 8080")
+        logger.warning(
+            "Disk space on drive '/var/log' is running low. Consider freeing up space"
+        )
+        logger.info("")
+
+        try:
+            raise Exception("Failed to connect to database: 'my_db'")
+        except Exception as e:
+            logger.info(f"=========start Exception============")
+            # exc_info=True ensures that a Traceback is included
+            FileLib.log_exception(e, logger)
+            logger.info(f"=========end Exception============")
+
+        print(f"FINISHED")
+
